==> speakrec-0.1.4.tar.gz: speakrec-0.1.4/lf30hz.cpp <==
/*
 * Speak Recorder - Low Frequency 30 Hz
 * Copyright  2004 by Schou Industries ApS.
 */

#include <iostream>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>  /* O_RDWR */
#include <string.h>
#include <errno.h>

#include "getoption.h"
#include "sinustable.inc"

using namespace std;

#define SAMPLERATE 44100
#define BUFSIZE (SAMPLERATE/10)
#define FRAMESIZE (sizeof(short))
#define BUFITEMS (BUFSIZE/FRAMESIZE)
#define BLOCKPRSEC ( (SAMPLERATE*FRAMESIZE) / BUFSIZE )

short sintable[SIZE_SINTABLE_1_4*4];
#define SIZE_SINTABLE (sizeof(sintable) / sizeof(sintable[0]))

int opt_help = 0;
int opt_verbose = 0;
int opt_version = 0;

int opt_lf_in = 2;
int opt_lf_out = 3;
int opt_lf_silence = 3;
int opt_ramp = BLOCKPRSEC / 2;
int opt_volume = 10;
int maxvolume = 32;

struct Get_long_options help_long_options[] = {
	{"help",       0, 0, 'h', &opt_help,       "\tThis help." },
	{"lf_in",      1, 0, 'i', &opt_lf_in,      "\tBlocks of LF in end of file." },
	{"lf_out",     1, 0, 'o', &opt_lf_out,     "\tBlocks of LF added to end of file." },
	{"lf_silence", 1, 0, 's', &opt_lf_silence, "\tBlocks of silence added to end of file." },
	{"volume",     1, 0, 'L', &opt_volume,     "\tVolume level. Default 10, maximum 32." },
	{"verbose",    0, 0, 'v', &opt_verbose,    "\tVerbose." },
	{"version",    0, 0, 'V', &opt_version,    "\tProgram version." },
	{0,0,0,0}
};

struct riff_hdr {
	char riff[4]; /* identifier string = "RIFF" */
	unsigned long remLen; /* remaining length after this header */
	char wave[4]; /* identifier string = "WAVE" */
	char fmt_[4]; /* identifier string = "fmt" */
	long           chunkSize;
	short          wFormatTag;
	unsigned short wChannels;
	unsigned long  dwSamplesPerSec;
	unsigned long  dwAvgBytesPerSec;
	unsigned short wBlockAlign;
	unsigned short wBitsPerSample;
	char id4[4]; /* data */
	long dataSize;
};

int br_eps = 0;
int br_dy = 32767 - 0; /* y2 - y1 */
int br_dx = opt_ramp * BUFITEMS - 0; /* x2 - x1 */
int br_y = 0; /* y1 */
int br_direction = 1; /* 1 = up, -1 = down */

int bresenham_y(void)
{

	br_eps += br_dy;
	if ( (br_eps << 1) >= br_dx ) {
		br_y += br_direction;
		br_eps -= br_dx;
	}

	return br_y;
}

/*
 * addfreq - returns the value to be added at current pos
 */

long idx_freq = 0;
int denominator = opt_ramp * BUFITEMS;

int addfreq(int value)
{
	int lf_value;

	lf_value = (sintable[idx_freq % SIZE_SINTABLE] * opt_volume) / maxvolume;
	if (idx_freq < opt_lf_in*BUFITEMS) {
		/* We are inside file */
		if (idx_freq < opt_ramp*BUFITEMS) {
			/* We are ramping up */
			lf_value = (lf_value * bresenham_y()) / denominator;
		} else {
			if (idx_freq == opt_ramp*BUFITEMS) {
				/* prepare for ramp down */
				br_direction = -1;
				br_eps = 0;
			}
		}
	} else {
		/* We are outside file */
		if (idx_freq > (opt_lf_in+opt_lf_out-opt_ramp)*BUFITEMS) {
			/* We are ramping down */
			lf_value = (lf_value * bresenham_y()) / denominator;
		}
	}
	idx_freq++;

	lf_value += value;
	if (lf_value > 32700) {
		lf_value = 32700;
	}
	if (lf_value < -32700) {
		lf_value = -32700;
	}

	return lf_value;
}

int hexdump(char *buf, int size) {
	int n = 0;
	while (n<size) {
		if (! (n % 16)) {
			printf("%04x  ", n);
		}
		printf("%02x ", (unsigned char)*buf);
		buf++;
		n++;
		if (! (n % 8)) {
			printf(" ");
		}
		if (! (n % 16)) {
			printf("\n");
		}
	}
	printf("\n");

	return size;
}

int dump4(char *d) {
	int n = 4;
	printf("dump4: '");
	while (n--) {
		if (isprint(*d))
			putchar(*d++);
		else
			printf("0x%02x ", *d++);
	}
	printf("'\n");
}

int wavdump(struct riff_hdr *h) {
	dump4((char *)&h->riff);
	printf("remLen: %d\n", h->remLen);
	dump4((char *)&h->wave);
	dump4((char *)&h->fmt_);
	printf("chunkSize: %d\n", h->chunkSize);
	printf("wFormatTag: %d\n", h->wFormatTag);
	printf("wChannels: %d\n", h->wChannels);
	printf("dwSamplesPerSec: %d\n", h->dwSamplesPerSec);
	printf("dwAvgBytesPerSec: %d\n", h->dwAvgBytesPerSec);
	printf("wBlockAlign: %d\n", h->wBlockAlign);
	printf("wBitsPerSample: %d\n", h->wBitsPerSample);
	dump4((char *)&h->id4);
	printf("dataSize: %d\n", h->dataSize);
}

int main(int argc, char **argv)
{
	struct riff_hdr header;
	char *filename;
	int fd;
	size_t filesize;
	short buf[BUFITEMS];
	int i, n;
	int value;

	GetOption getoption(argc, argv, help_long_options);
	if (opt_help) {
 		cout << "Copyright  2004 by Schou Industries ApS." << endl;
		getoption.help(fileno(stdout));
		cout << endl;
		cout << "http://schou.dk/ http://www.speltron.dk/" << endl;
		return 0;
	}
	br_dx = opt_ramp * BUFITEMS - 0; /* x2 - x1 */
	opt_lf_in *= BLOCKPRSEC;
	opt_lf_out *= BLOCKPRSEC;
	opt_lf_silence *= BLOCKPRSEC;

	for (i=0; i<SIZE_SINTABLE_1_4; i++) {
		sintable[i]                       =      sintable_1_4[i];
		sintable[SIZE_SINTABLE_1_4*2-i-1] =      sintable_1_4[i];
		sintable[SIZE_SINTABLE_1_4*2+i]   = -1 * sintable_1_4[i];
		sintable[SIZE_SINTABLE_1_4*4-i-1] = -1 * sintable_1_4[i];
	}

	filename = argv[argc-1];
	if (opt_verbose) {
		printf("Open file: %s\n", filename);
		printf("Blocks per sec: %d\n", BLOCKPRSEC);
		printf("Frames per block: %d\n", BUFITEMS);
		printf("Volume: %d\n", opt_volume);
	}
	
	fd = open(filename, O_RDWR);
	if (fd < 1) {
		printf("open error: %s\n", strerror(errno));
	} else {
		i = read(fd, &header, sizeof(header));
		if (opt_verbose) {
			wavdump(&header);
		}
		header.dataSize += (opt_lf_out + opt_lf_silence) * BUFSIZE;
		if (lseek(fd, 0, SEEK_SET) < 0) {
			printf("lseek error: %s\n", strerror(errno));
		}
		i = write(fd, &header, sizeof(header));

		/* Add LF to the last part of file */
		lseek(fd, BUFSIZE*opt_lf_in*-1, SEEK_END);
		i = opt_lf_in;
		while (i--) {
			read(fd, &buf, sizeof(buf));
			n = 0;
			while (n < BUFITEMS) {
				buf[n] = addfreq(buf[n]);
				n++;
			}
			lseek(fd, -sizeof(buf), SEEK_CUR);
			write(fd, &buf, sizeof(buf));
		}

		/* Add LF to end of file */
		i = opt_lf_out;
		while (i--) {
			n = 0;
			while (n < BUFITEMS) {
				buf[n] = addfreq(0);
				n++;
			}
			write(fd, &buf, sizeof(buf));
		}

		/* Add silence to end of file */
		bzero(&buf, sizeof(buf));
		i = opt_lf_silence;
		while (i--) {
			write(fd, &buf, sizeof(buf));
		}

		close(fd);
	}


	return 0;
}

==> speakrec-0.1.6.tar.gz: speakrec-0.1.6/lf30hz.cpp <==
/*
 * Speak Recorder - Low Frequency 30 Hz
 * Copyright  2004 by Schou Industries ApS.
 */

#include <iostream>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>  /* O_RDWR */
#include <string.h>
#include <errno.h>

#include "getoption.h"
#include "sinustable.inc"

using namespace std;

#define SAMPLERATE 44100
#define BUFSIZE (SAMPLERATE/10)
#define FRAMESIZE (sizeof(short))
#define BUFITEMS (unsigned int)(BUFSIZE/FRAMESIZE)
#define BLOCKPRSEC ( (SAMPLERATE*FRAMESIZE) / BUFSIZE )

short sintable[SIZE_SINTABLE_1_4*4];
#define SIZE_SINTABLE (unsigned int)((sizeof(sintable) / sizeof(sintable[0])))

int opt_help = 0;
int opt_verbose = 0;
int opt_version = 0;

int opt_lf_in = 2;
int opt_lf_out = 3;
int opt_lf_silence = 3;
int opt_ramp = BLOCKPRSEC / 2;
int opt_volume = 10;
int maxvolume = 32;

struct Get_long_options help_long_options[] = {
	{"help",       t_non, 0, 'h', &opt_help,       "\tThis help." },
	{"lf_in",      t_int, 0, 'i', &opt_lf_in,      "\tBlocks of LF in end of file." },
	{"lf_out",     t_int, 0, 'o', &opt_lf_out,     "\tBlocks of LF added to end of file." },
	{"lf_silence", t_int, 0, 's', &opt_lf_silence, "\tBlocks of silence added to end of file." },
	{"volume",     t_int, 0, 'L', &opt_volume,     "\tVolume level. Default 10, maximum 32." },
	{"verbose",    t_non, 0, 'v', &opt_verbose,    "\tVerbose." },
	{"version",    t_non, 0, 'V', &opt_version,    "\tProgram version." },
	{0,t_non,0,0}
};

struct riff_hdr {
	uint8_t  riff[4];          /* 4 identifier string = "RIFF" */
	uint32_t remLen;           /* 4 remaining length after this header */
	uint8_t  wave[4];          /* 4 identifier string = "WAVE" */
	uint8_t  fmt_[4];          /* 4 identifier string = "fmt" */
	uint32_t chunkSize;        /* 4 */
	uint16_t wFormatTag;       /* 2 */
	uint16_t wChannels;        /* 2 */
	uint32_t dwSamplesPerSec;  /* 4 */
	uint32_t dwAvgBytesPerSec; /* 4 */
	uint16_t wBlockAlign;      /* 2 */
	uint16_t wBitsPerSample;   /* 2 */
	uint8_t  id4[4];           /* 4 data */
	uint32_t dataSize;         /* 4 */
};

int br_eps = 0;
int br_dy = 32767 - 0; /* y2 - y1 */
int br_dx = opt_ramp * BUFITEMS - 0; /* x2 - x1 */
int br_y = 0; /* y1 */
int br_direction = 1; /* 1 = up, -1 = down */

int bresenham_y(void)
{

	br_eps += br_dy;
	if ( (br_eps << 1) >= br_dx ) {
		br_y += br_direction;
		br_eps -= br_dx;
	}

	return br_y;
}

/*
 * addfreq - returns the value to be added at current pos
 */

long idx_freq = 0;
int denominator = opt_ramp * BUFITEMS;

int addfreq(int value)
{
	int lf_value;

	lf_value = (sintable[idx_freq % SIZE_SINTABLE] * opt_volume) / maxvolume;
	if (idx_freq < opt_lf_in*BUFITEMS) {
		/* We are inside file */
		if (idx_freq < opt_ramp*BUFITEMS) {
			/* We are ramping up */
			lf_value = (lf_value * bresenham_y()) / denominator;
		} else {
			if (idx_freq == opt_ramp*BUFITEMS) {
				/* prepare for ramp down */
				br_direction = -1;
				br_eps = 0;
			}
		}
	} else {
		/* We are outside file */
		if (idx_freq > (opt_lf_in+opt_lf_out-opt_ramp)*BUFITEMS) {
			/* We are ramping down */
			lf_value = (lf_value * bresenham_y()) / denominator;
		}
	}
	idx_freq++;

	lf_value += value;
	if (lf_value > 32700) {
		lf_value = 32700;
	}
	if (lf_value < -32700) {
		lf_value = -32700;
	}

	return lf_value;
}

int hexdump(char *buf, int size) {
	int n = 0;
	while (n<size) {
		if (! (n % 16)) {
			printf("%04x  ", n);
		}
		printf("%02x ", (unsigned char)*buf);
		buf++;
		n++;
		if (! (n % 8)) {
			printf(" ");
		}
		if (! (n % 16)) {
			printf("\n");
		}
	}
	printf("\n");

	return size;
}

int dump4(char *d) {
	int n = 4;
	printf("dump4: '");
	while (n--) {
		if (isprint(*d))
			putchar(*d++);
		else
			printf("0x%02x ", *d++);
	}
	printf("'\n");

	return 0;
}

int wavdump(struct riff_hdr *h) {
	dump4((char *)&h->riff);
	printf("remLen: %d\n", h->remLen);
	dump4((char *)&h->wave);
	dump4((char *)&h->fmt_);
	printf("chunkSize: %d\n", h->chunkSize);
	printf("wFormatTag: %d\n", h->wFormatTag);
	printf("wChannels: %d\n", h->wChannels);
	printf("dwSamplesPerSec: %d\n", h->dwSamplesPerSec);
	printf("dwAvgBytesPerSec: %d\n", h->dwAvgBytesPerSec);
	printf("wBlockAlign: %d\n", h->wBlockAlign);
	printf("wBitsPerSample: %d\n", h->wBitsPerSample);
	dump4((char *)&h->id4);
	printf("dataSize: %d\n", h->dataSize);

	return 0;
}

int main(int argc, char **argv)
{
	struct riff_hdr header;
	char *filename;
	int fd;
	short buf[BUFITEMS];
	unsigned int i, n;

	GetOption getoption(argc, argv, help_long_options);
	if (opt_help) {
 		cout << "Copyright  2004 by Schou Industries ApS." << endl;
		getoption.help(fileno(stdout));
		cout << endl;
		cout << "http://schou.dk/ http://www.speltron.dk/" << endl;
		return 0;
	}
	br_dx = opt_ramp * BUFITEMS - 0; /* x2 - x1 */
	opt_lf_in *= BLOCKPRSEC;
	opt_lf_out *= BLOCKPRSEC;
	opt_lf_silence *= BLOCKPRSEC;

	for (i=0; i<SIZE_SINTABLE_1_4; i++) {
		sintable[i]                       =      sintable_1_4[i];
		sintable[SIZE_SINTABLE_1_4*2-i-1] =      sintable_1_4[i];
		sintable[SIZE_SINTABLE_1_4*2+i]   = -1 * sintable_1_4[i];
		sintable[SIZE_SINTABLE_1_4*4-i-1] = -1 * sintable_1_4[i];
	}

	filename = argv[argc-1];
	if (opt_verbose) {
		printf("Open file: %s\n", filename);
		printf("Blocks per sec: %lu\n", BLOCKPRSEC);
		printf("Frames per block: %d\n", BUFITEMS);
		printf("Volume: %d\n", opt_volume);
	}
	
	fd = open(filename, O_RDWR);
	if (fd < 1) {
		printf("open error: %s\n", strerror(errno));
	} else {
		i = read(fd, &header, sizeof(header));
		if (opt_verbose) {
			wavdump(&header);
		}
		header.dataSize += (opt_lf_out + opt_lf_silence) * BUFSIZE;
		if (lseek(fd, 0, SEEK_SET) < 0) {
			printf("lseek error: %s\n", strerror(errno));
		}
		i = write(fd, &header, sizeof(header));

		/* Add LF to the last part of file */
		lseek(fd, BUFSIZE*opt_lf_in*-1, SEEK_END);
		i = opt_lf_in;
		while (i--) {
			read(fd, &buf, sizeof(buf));
			n = 0;
			while (n < BUFITEMS) {
				buf[n] = addfreq(buf[n]);
				n++;
			}
			lseek(fd, -sizeof(buf), SEEK_CUR);
			write(fd, &buf, sizeof(buf));
		}

		/* Add LF to end of file */
		i = opt_lf_out;
		while (i--) {
			n = 0;
			while (n < BUFITEMS) {
				buf[n] = addfreq(0);
				n++;
			}
			write(fd, &buf, sizeof(buf));
		}

		/* Add silence to end of file */
		bzero(&buf, sizeof(buf));
		i = opt_lf_silence;
		while (i--) {
			write(fd, &buf, sizeof(buf));
		}

		close(fd);
	}


	return 0;
}
